// example #6;
// here we connect an 8x8 LED matrix to Arduino, which also has connected
// an ESP8266 wifi module; the LED matrix displays a heart shape with a 
// lighting effect that can be one of: heart beating or heart fading;
// either of these effects can be selected via an Android app with two
// buttons;
//
// connections of the 8x8 LED matrix:
// MAX7219 VCC pin > Arduino 5V pin
// MAX7219 GND pin > Arduino GND pin
// MAX7219 DIN pin > Arduino pin 2
// MAX7219 CS pin > Arduino pin 3
// MAX7219 CLOCK pin > Arduino pin 4
//
// connections to the ESP8266 are done via 3-resistor level converter;
// ESP8266 TX > Arduino pin 2 (used as RX of Software serial)
// ESP8266 RX > 3-resistor > Arduino pin 3 (used as TX of Software serial)


#include <SoftwareSerial.h>
 
#define DEBUG true


int ANIMDELAY = 100;  // animation delay, deafault value is 100
int INTENSITYMIN = 0; // minimum brightness, valid range [0,15]
int INTENSITYMAX = 8; // maximum brightness, valid range [0,15]

int DIN_PIN = 2;      // data in pin
int CS_PIN = 3;       // load (CS) pin
int CLK_PIN = 4;      // clock pin

// MAX7219 registers
byte MAXREG_DECODEMODE = 0x09;
byte MAXREG_INTENSITY  = 0x0a;
byte MAXREG_SCANLIMIT  = 0x0b;
byte MAXREG_SHUTDOWN   = 0x0c;
byte MAXREG_DISPTEST   = 0x0f;

const unsigned char heart[] =
{
  B01100110,
  B11111111,
  B11111111,
  B11111111,
  B01111110,
  B00111100,
  B00011000,
  B00000000
};


boolean turn_ON_OFF_beating = false;
boolean turn_ON_OFF_fading = false;
int effectNumber = 0;


// make RX Arduino line is pin 10, make TX Arduino line is pin 11.
// This means that you need to connect the TX line from the esp to the Arduino's pin 10
// and the RX line from the esp to the Arduino's pin 11
SoftwareSerial esp8266(10,11);

void setup()
{

  // (1) stuff related to ESP8266 and Arduino's connection to host PC; 
  // hardware serial connects to host PC; software serial connects to ESP8266;
  
  Serial.begin(9600);
  esp8266.begin(9600); // your esp's baud rate might be different;

  // method 1:
  // use name and pass; this is doable at home, where you simply can connect
  // to yoru own router with user name and pass;
  /*---
  sendCommand("AT+RST\r\n",2000,DEBUG); // reset module
  sendCommand("AT+CWMODE=1\r\n",1000,DEBUG); // configure as access point
  sendCommand("AT+CWJAP=\"OshkoshWolf\",\"YourPasswordHere\"\r\n",3000,DEBUG);
  delay(10000);
  sendCommand("AT+CIFSR\r\n",1000,DEBUG); // get ip address
  sendCommand("AT+CIPMUX=1\r\n",1000,DEBUG); // configure for multiple connections
  sendCommand("AT+CIPSERVER=1,80\r\n",1000,DEBUG); // turn on server on port 80
  ---*/
  // method 2:
  // does not use name and pass; but, you will have to check wifi networks
  // available on your Android phone and connect to one that is the ESP8266;
  // on the Android phone, when using the provided app, you must type in the IP
  // address of the ESP8266 module, as printed by the debug messages on the Arduino's
  // Serial terminal window;  
  sendData("AT+RST\r\n",2000,DEBUG); // reset module
  sendData("AT+CWMODE=2\r\n",1000,DEBUG); // configure as access point
  sendData("AT+CIFSR\r\n",1000,DEBUG); // get ip address
  sendData("AT+CIPMUX=1\r\n",1000,DEBUG); // configure for multiple connections
  sendData("AT+CIPSERVER=1,80\r\n",1000,DEBUG); // turn on server on port 80

  
  Serial.println("Server on ESP8266 module ready!");  
  
  // (2) things related to the 8x8 LED matrix display;
  
  pinMode(DIN_PIN, OUTPUT);
  pinMode(CLK_PIN, OUTPUT);
  pinMode(CS_PIN, OUTPUT);

  // initialization of the MAX7219;
  setRegistry(MAXREG_SCANLIMIT, 0x07);
  setRegistry(MAXREG_DECODEMODE, 0x00);  // using an led matrix (not digits)
  setRegistry(MAXREG_SHUTDOWN, 0x01);    // not in shutdown mode
  setRegistry(MAXREG_DISPTEST, 0x00);    // no display test
  setRegistry(MAXREG_INTENSITY, 0x0f & INTENSITYMIN);

  // draw heart;
  heart_static_and_dim(); 
}


void loop()
{

  // (1) check on ESP8266; did it receive anything?
  if (esp8266.available()) // check if the esp is sending a message to the Arduino;
  {    
    if (esp8266.find("+IPD,"))
    {
      delay(2000); // wait for the serial buffer to fill up (read all the serial data)

      // get the connection id so that we can then disconnect
      // subtract 48 because the read() function returns 
      // the ASCII decimal value and 0 (the first decimal number) starts at 48
      int connectionId = esp8266.read()-48;
          
      esp8266.find("effect="); // advance cursor to "effect="
                
      // get effect number, which requires to action upon;
      effectNumber = -1;
      effectNumber = (esp8266.read()-48);

      // take appropriate action according to received effect number;
      // for now it can be 1 or 2; else do nothing;
      if ( effectNumber == 1) {
        //turn_ON_OFF_beating = !turn_ON_OFF_beating; // if want toggle;
        turn_ON_OFF_beating = true;
        turn_ON_OFF_fading = false;
      } else if ( effectNumber == 2) {
        //turn_ON_OFF_fading = !turn_ON_OFF_fading; // if want toggle;
        turn_ON_OFF_beating = false;
        turn_ON_OFF_fading = true;
      } else {
        turn_ON_OFF_beating = false;
        turn_ON_OFF_fading = false;
      }
      
      // build string that is send back to device that is requesting effect toggle;
      String content;
      if ( effectNumber == 1 || effectNumber == 2) {
        content = "Effect ";
        content += effectNumber;
        content += " is ";
        content += "ON";
      } else {
        content = "Comm err! Effects OFF";
      }

      sendHTTPResponse(connectionId, content);

      // make close command
      String closeCommand = "AT+CIPCLOSE="; 
      closeCommand+=connectionId; // append connection id
      closeCommand+="\r\n";

      sendCommand(closeCommand,1000,DEBUG); // close connection
    }
  }
  
  // (2) toggle effects;
  if ( turn_ON_OFF_beating == true) {
    heart_beating_effect();
  } else if ( turn_ON_OFF_fading == true) {
    heart_fading_effect();
  } else {
    heart_static_and_dim();
  }
  
}

///////////////////////////////////////////////////////////////////////////////
//
// functions related to handling the ESP8266 wifi module;
//
///////////////////////////////////////////////////////////////////////////////

// Name: sendHTTPResponse
// Description: Function that sends HTTP 200, HTML UTF-8 response
void sendHTTPResponse(int connectionId, String content)
{    
     // build HTTP response
     String httpResponse;
     String httpHeader;
     // HTTP Header
     httpHeader = "HTTP/1.1 200 OK\r\nContent-Type: text/html; charset=UTF-8\r\n"; 
     httpHeader += "Content-Length: ";
     httpHeader += content.length();
     httpHeader += "\r\n";
     httpHeader +="Connection: close\r\n\r\n";
       
     httpResponse = httpHeader + content;
     sendCIPData(connectionId, httpResponse);
}
 
// Name: sendCIPDATA
// Description: sends a CIPSEND=<connectionId>,<data> command
void sendCIPData(int connectionId, String data)
{
   String cipSend = "AT+CIPSEND=";
   cipSend += connectionId;
   cipSend += ",";
   cipSend +=data.length();
   cipSend +="\r\n";
   sendCommand(cipSend,1000,DEBUG);
   sendData(data,1000,DEBUG);
}
 
// Name: sendCommand
// Description: Function used to send data to ESP8266.
// Params: command - the data/command to send; timeout - the time to wait for a response; 
//         debug - print to Serial window? (true = yes, false = no)
// Returns: The response from the esp8266 (if there is a reponse)
String sendCommand(String command, const int timeout, boolean debug)
{
    String response = "";
           
    esp8266.print(command); // send the read character to the esp8266
    
    long int time = millis();   
    while ( (time+timeout) > millis())
    {
      while (esp8266.available())
      {      
        // The esp has data so display its output to the serial window 
        char c = esp8266.read(); // read the next character
        response+=c;
      }  
    }
    
    if (debug) {
      Serial.print(response);
    }   
    return response;
}

// Name: sendData
// Description: Function used to send data to ESP8266
// Params: command - the data/command to send; timeout - the time to wait for a response;
//         debug - print to Serial window? (true = yes, false = no)
// Returns: The response from the esp8266 (if there is a reponse)
String sendData(String command, const int timeout, boolean debug)
{
    String response = "";
    
    int dataSize = command.length() + 1; // why +1?
    char data[dataSize];
    command.toCharArray(data,dataSize);
           
    esp8266.write(data,dataSize); // send the read character to the esp8266
    if (debug) {
      Serial.println("\r\n====== HTTP Response From Arduino ======");
      Serial.write(data,dataSize);
      Serial.println("\r\n========================================");
    }
    
    long int time = millis();    
    while ( (time+timeout) > millis())
    {
      while (esp8266.available())
      {
        // The esp has data so display its output to the serial window 
        char c = esp8266.read(); // read the next character.
        response+=c;
      }  
    }
    
    if (debug) {
      Serial.print(response);
    }   
    return response;
}

///////////////////////////////////////////////////////////////////////////////
//
// functions related to handling the 8x8 LED matrix;
//
///////////////////////////////////////////////////////////////////////////////

void heart_beating_effect()
{
  // first beat
  setRegistry(MAXREG_INTENSITY, 0x0f & INTENSITYMAX);
  delay(ANIMDELAY); 
  // switch off
  setRegistry(MAXREG_INTENSITY, 0x0f & INTENSITYMIN);
  delay(ANIMDELAY);  
  // second beat
  setRegistry(MAXREG_INTENSITY, 0x0f & INTENSITYMAX);
  delay(ANIMDELAY); 
  // switch off
  setRegistry(MAXREG_INTENSITY, 0x0f & INTENSITYMIN);
  delay(ANIMDELAY*6);
}

void heart_fading_effect()
{
  for (int i = 15; i > 0; i--) {
    setRegistry(MAXREG_INTENSITY, 0x0f & i);
    delay(200); 
  }
}

void heart_static_and_dim()
{
  setRegistry(MAXREG_INTENSITY, 0x0f & INTENSITYMIN);
  setRegistry(1, heart[0]);
  setRegistry(2, heart[1]);
  setRegistry(3, heart[2]);
  setRegistry(4, heart[3]);
  setRegistry(5, heart[4]);
  setRegistry(6, heart[5]);
  setRegistry(7, heart[6]);
  setRegistry(8, heart[7]);
}

void setRegistry(byte reg, byte value)
{
  digitalWrite(CS_PIN, LOW);

  putByte(reg);   // specify register
  putByte(value); // send data

  digitalWrite(CS_PIN, LOW);
  digitalWrite(CS_PIN, HIGH);
}

void putByte(byte data)
{
  byte i = 8;
  byte mask;
  while (i > 0)
  {
    mask = 0x01 << (i - 1);        // get bitmask
    digitalWrite( CLK_PIN, LOW);   // tick
    if (data & mask)               // choose bit
      digitalWrite(DIN_PIN, HIGH); // send 1
    else
      digitalWrite(DIN_PIN, LOW);  // send 0
    digitalWrite(CLK_PIN, HIGH);   // tock
    --i;                           // move to lesser bit
  }
}

